@@ -4,31 +4,6 @@
 
 .onLoad <- function(...) {
   ensureInitialized()
-  saveNframesCallback(parse(text="later:::nframe()")[[1]])
-}
-
-# I don't know why, but it's necessary to wrap sys.nframe (an .Internal) inside 
-# a regular function call. Otherwise, you get the wrong results in the following
-# situation:
-#
-# a <- function() {
-#   later::later(~print("Got here 2"))
-#   Sys.sleep(1)
-#   print("Got here 1")
-# }
-# a()
-#
-# (To repro you must execute this in the R console, not source a script that 
-# contains it, as the source() call itself will ruin the repro)
-#
-# The desired behavior is to sleep 1 second, then print "Got here 1", the print 
-# "Got here 2". But if sys.nframe() is passed directly to saveNframesCallback,
-# you get "Got here 2", then sleep 1 second, then "Got here 1".
-#
-# I spent half a day with a debug version of R and gdb trying to get to the root
-# cause, without success.
-nframe <- function() {
-  sys.nframe()-1
 }
 
 #' Executes a function later
@@ -91,11 +66,14 @@ later <- function(func, delay = 0) {
 #'   an operation to be ready to run. If `0`, then return immediately if there
 #'   are no operations that are ready to run. If `Inf` or negative, then wait as
 #'   long as it takes (if none are scheduled, then this will block forever).
-#'
+#' @param all If `FALSE`, `run_now()` will execute at most one scheduled
+#'   operation (instead of all eligible operations). This can be useful in cases
+#'   where you want to interleave scheduled operations with your own logic.
+#'   
 #' @return A logical indicating whether any callbacks were actually run.
 #'
 #' @export
-run_now <- function(timeoutSecs = 0L) {
+run_now <- function(timeoutSecs = 0L, all = TRUE) {
   if (timeoutSecs == Inf) {
     timeoutSecs <- -1
   }
@@ -103,7 +81,7 @@ run_now <- function(timeoutSecs = 0L) {
   if (!is.numeric(timeoutSecs))
     stop("timeoutSecs must be numeric")
   
-  invisible(execCallbacks(timeoutSecs))
+  invisible(execCallbacks(timeoutSecs, all))
 }
 
 #' Check if later loop is empty
